import javax.microedition.lcdui.*;
import java.util.Vector;

public class TranslatedList extends List implements CommandListener  
{ 
    Dictionary dictionary;
    TranslatedWord[] words;
    TranslatedWord[] wordsByTime;
    TranslatedWord[] wordsByFreq;

    TranslatedList(Dictionary dictionary) 
    {
        super(Locale.HISTORY, IMPLICIT);
        this.dictionary = dictionary;        
        wordsByTime = new TranslatedWord[dictionary.translatedWords.size()];
        int[] freq = new int[256];
        int n = wordsByTime.length;
        for (int i = 0; i < n; i++) { 
            TranslatedWord tw = (TranslatedWord)dictionary.translatedWords.elementAt(i);
            append(tw.word, null);
            freq[tw.frequency] += 1;
            wordsByTime[i] = tw;
        }
        int offs = 0;
        for (int i = freq.length; --i >= 0;) {
            int f = freq[i];
            freq[i] = offs;
            offs += f;
        }
        words = wordsByTime;
        wordsByFreq = new TranslatedWord[n];
        for (int i = 0; i < n; i++) { 
            wordsByFreq[freq[words[i].frequency]++] = words[i];
        }
        setCommandListener(this);
        addCommand(Dictionary.BACK_CMD);
        addCommand(Dictionary.ORDER_BY_FREQUENCY_CMD);
        Display.getDisplay(dictionary).setCurrent(this);
    }

    private void resetList() {
        for (int i = 0, n = words.length; i < n; i++) { 
            set(i, words[i].word, null);
        }
    }

    public void commandAction(Command c, Displayable d)  
    {
        if (c == Dictionary.BACK_CMD) {
            Display.getDisplay(dictionary).setCurrent(dictionary.mainMenu);
        } else if (c == Dictionary.ORDER_BY_FREQUENCY_CMD) {
            removeCommand(Dictionary.ORDER_BY_FREQUENCY_CMD);
            addCommand(Dictionary.ORDER_BY_TIME_CMD);
            words = wordsByFreq;
            resetList();
        } else if (c == Dictionary.ORDER_BY_TIME_CMD) {
            addCommand(Dictionary.ORDER_BY_FREQUENCY_CMD);
            removeCommand(Dictionary.ORDER_BY_TIME_CMD);
            words = wordsByTime;
            resetList();
        } else { 
            int i = getSelectedIndex();
            if (i >= 0) { 
                Vector selection = dictionary.getSelection(words[i].word);
                new TranslationBox(dictionary, this, (Translation)selection.elementAt(0));
            }
        }           
    }
}                  
